@@ -23,9 +23,9 @@ def format_error(
 
     location = ""
     if lineno and snakefile:
-        location = f"in line {lineno} of {snakefile}"
+        location = f" in file {snakefile}, line {lineno}"
         if rule:
-            location = f" in rule {rule} {location}"
+            location = f" in rule {rule}{location}"
 
     tb = ""
     if show_traceback:
@@ -292,9 +292,9 @@ class MissingOutputException(RuleException):
         rule=None,
         jobid="",
     ):
-        message = "Job {} completed successfully, but some output files are missing. {}".format(
-            message, jobid
-        )
+        if jobid:
+            jobid = f"{jobid} "
+        message = f"Job {jobid} completed successfully, but some output files are missing. {message}"
         super().__init__(message, include, lineno, snakefile, rule)
 
 
