@@ -61,7 +61,7 @@ def validate(data, schema, set_default=True):
 
     if isinstance(schemafile, LocalSourceFile) and not schemafile.isabs() and workflow:
         # if workflow object is not available this has not been started from a workflow
-        schemafile = workflow.current_basedir.join(schemafile)
+        schemafile = workflow.current_basedir.join(schemafile.get_path_or_uri())
 
     source = (
         workflow.sourcecache.open(schemafile)
@@ -430,7 +430,10 @@ def format(_pattern, *args, stepout=1, _quote_all=False, quote_func=None, **kwar
     try:
         return fmt.format(_pattern, *args, **variables)
     except KeyError as ex:
-        if str(ex).strip("'") in variables["wildcards"].keys():
+        if (
+            "wildcards" in variables
+            and str(ex).strip("'") in variables["wildcards"].keys()
+        ):
             raise NameError(
                 "The name '{0}' is unknown in this context. "
                 "Did you mean 'wildcards.{0}'?".format(str(ex).strip("'"))
